@@ -53,6 +53,7 @@ struct XfpmBrightnessPrivate
 
   gint32    max_level;
   gint32    current_level;
+  gint32    hw_min_level;
   gint32    min_level;
   gint32    step;
   gfloat    exp_step;
@@ -60,30 +61,46 @@ struct XfpmBrightnessPrivate
 
 G_DEFINE_TYPE_WITH_PRIVATE (XfpmBrightness, xfpm_brightness, G_TYPE_OBJECT)
 
-static gint32
-xfpm_brightness_inc (XfpmBrightness *brightness, gint32 level)
+gboolean
+xfpm_brightness_increase (XfpmBrightness *brightness)
 {
-  if (brightness->priv->use_exp_step)
+  gint32 level, new_level;
+
+  if (!xfpm_brightness_get_level (brightness, &level))
+    return FALSE;
+
+  if ( brightness->priv->use_exp_step )
   {
-    gint32 new_level = roundf (level * brightness->priv->exp_step);
-    if (new_level == level)  ++new_level;
-    return new_level;
+    new_level = roundf (level * brightness->priv->exp_step);
+    if (new_level == level) ++new_level;
   }
   else
-    return level + brightness->priv->step;
+  {
+    new_level = level + brightness->priv->step;
+  }
+
+  return xfpm_brightness_set_level (brightness, new_level);
 }
 
-static gint32
-xfpm_brightness_dec (XfpmBrightness *brightness, gint32 level)
+gboolean
+xfpm_brightness_decrease (XfpmBrightness *brightness)
 {
+  gint32 level, new_level;
+
+  if (!xfpm_brightness_get_level (brightness, &level))
+    return FALSE;
+
   if (brightness->priv->use_exp_step)
   {
-    gint32 new_level = roundf (level / brightness->priv->exp_step);
-    if (new_level == level)  --new_level;
-    return new_level;
+    new_level = roundf (level / brightness->priv->exp_step);
+    if (new_level == level) --new_level;
   }
   else
-    return level - brightness->priv->step;
+  {
+    new_level = level - brightness->priv->step;
+  }
+
+  return xfpm_brightness_set_level (brightness, new_level);
 }
 
 static gboolean
@@ -220,7 +237,7 @@ xfpm_brightness_setup_xrandr (XfpmBrightness *brightness)
 
   if (brightness->priv->backlight == None)
   {
-    g_warning ("No outputs have backlight property");
+    XFPM_DEBUG ("No outputs have backlight property");
     return FALSE;
   }
 
@@ -264,85 +281,7 @@ xfpm_brightness_setup_xrandr (XfpmBrightness *brightness)
   return ret;
 }
 
-static gboolean
-xfpm_brightness_xrand_up (XfpmBrightness *brightness, gint32 *new_level)
-{
-  gint32 hw_level;
-  gboolean ret = FALSE;
-  gint32 set_level;
 
-  ret = xfpm_brightness_xrandr_get_level (brightness, brightness->priv->output, &hw_level);
-
-  if ( !ret )
-    return FALSE;
-
-  if ( hw_level == brightness->priv->max_level )
-  {
-    *new_level = brightness->priv->max_level;
-    return TRUE;
-  }
-
-  set_level = MIN (xfpm_brightness_inc (brightness, hw_level), brightness->priv->max_level);
-
-  g_warn_if_fail (xfpm_brightness_xrandr_set_level (brightness, brightness->priv->output, set_level));
-
-  ret = xfpm_brightness_xrandr_get_level (brightness, brightness->priv->output, new_level);
-
-  if ( !ret )
-  {
-    g_warning ("xfpm_brightness_xrand_up failed for %d", set_level);
-    return FALSE;
-  }
-
-  /* Nothing changed in the hardware*/
-  if ( *new_level == hw_level )
-  {
-    g_warning ("xfpm_brightness_xrand_up did not change the hw level to %d", set_level);
-    return FALSE;
-  }
-
-  return TRUE;
-}
-
-static gboolean
-xfpm_brightness_xrand_down (XfpmBrightness *brightness, gint32 *new_level)
-{
-  gint32 hw_level;
-  gboolean ret;
-  gint32 set_level;
-
-  ret = xfpm_brightness_xrandr_get_level (brightness, brightness->priv->output, &hw_level);
-
-  if ( !ret )
-    return FALSE;
-
-  if ( hw_level == brightness->priv->min_level )
-  {
-    *new_level = brightness->priv->min_level;
-    return TRUE;
-  }
-
-  set_level = MAX (xfpm_brightness_dec (brightness, hw_level), brightness->priv->min_level);
-
-  g_warn_if_fail (xfpm_brightness_xrandr_set_level (brightness, brightness->priv->output, set_level));
-
-  ret = xfpm_brightness_xrandr_get_level (brightness, brightness->priv->output, new_level);
-
-  if ( !ret )
-  {
-    g_warning ("xfpm_brightness_xrand_down failed for %d", set_level);
-    return FALSE;
-  }
-
-  /* Nothing changed in the hardware*/
-  if ( *new_level == hw_level )
-  {
-    g_warning ("xfpm_brightness_xrand_down did not change the hw level to %d", set_level);
-    return FALSE;
-  }
-
-  return TRUE;
-}
 
 /*
  * Non-XRandR fallback using xfpm-backlight-helper
@@ -353,26 +292,21 @@ xfpm_brightness_xrand_down (XfpmBrightness *brightness, gint32 *new_level)
 static gint
 xfpm_brightness_helper_get_value (const gchar *argument)
 {
-  gboolean ret;
   GError *error = NULL;
   gchar *stdout_data = NULL;
-  gint exit_status = 0;
+  gint status;
   gint value = -1;
-  gchar *command = NULL;
+  gchar *command = g_strdup_printf (SBINDIR "/xfpm-power-backlight-helper --%s", argument);
 
-  command = g_strdup_printf (SBINDIR "/xfpm-power-backlight-helper --%s", argument);
-  ret = g_spawn_command_line_sync (command,
-                                   &stdout_data, NULL, &exit_status, &error);
-  if ( !ret )
+  XFPM_DEBUG ("Executing command: %s", command);
+  if (!g_spawn_command_line_sync (command, &stdout_data, NULL, &status, &error)
+      || !g_spawn_check_exit_status (status, &error))
   {
-    g_warning ("failed to get value: %s", error->message);
+    XFPM_DEBUG ("Failed to get value: %s", error->message);
     g_error_free (error);
-    goto out;
+    g_free (command);
+    return value;
   }
-  g_debug ("executed %s; retval: %i", command, exit_status);
-
-  if ( exit_status != 0 )
-    goto out;
 
 #if !defined(BACKEND_TYPE_FREEBSD)
   if ( stdout_data[0] == 'N' )
@@ -385,7 +319,6 @@ xfpm_brightness_helper_get_value (const gchar *argument)
   value = atoi (stdout_data);
 #endif
 
-out:
   g_free (command);
   g_free (stdout_data);
   return value;
@@ -397,7 +330,7 @@ xfpm_brightness_setup_helper (XfpmBrightness *brightness)
   gint32 ret;
 
   ret = (gint32) xfpm_brightness_helper_get_value ("get-max-brightness");
-  g_debug ("xfpm_brightness_setup_helper: get-max-brightness returned %i", ret);
+  XFPM_DEBUG ("xfpm_brightness_setup_helper: get-max-brightness returned %i", ret);
   if ( ret < 0 )
   {
     brightness->priv->helper_has_hw = FALSE;
@@ -424,7 +357,7 @@ xfpm_brightness_helper_get_level (XfpmBrightness *brg, gint32 *level)
 
   ret = (gint32) xfpm_brightness_helper_get_value ("get-brightness");
 
-  g_debug ("xfpm_brightness_helper_get_level: get-brightness returned %i", ret);
+  XFPM_DEBUG ("xfpm_brightness_helper_get_level: get-brightness returned %i", ret);
 
   if ( ret >= 0 )
   {
@@ -438,25 +371,22 @@ xfpm_brightness_helper_get_level (XfpmBrightness *brg, gint32 *level)
 static gboolean
 xfpm_brightness_helper_set_level (XfpmBrightness *brg, gint32 level)
 {
-  gboolean ret;
   GError *error = NULL;
-  gint exit_status = 0;
-  gchar *command = NULL;
+  gint status;
+  gchar *command = g_strdup_printf ("pkexec " SBINDIR "/xfpm-power-backlight-helper --set-brightness %i", level);
 
-  command = g_strdup_printf ("pkexec " SBINDIR "/xfpm-power-backlight-helper --set-brightness %i", level);
-  ret = g_spawn_command_line_sync (command, NULL, NULL, &exit_status, &error);
-  if ( !ret )
+  XFPM_DEBUG ("Executing command: %s", command);
+  if (!g_spawn_command_line_sync (command, NULL, NULL, &status, &error)
+      || !g_spawn_check_exit_status (status, &error))
   {
-    g_warning ("xfpm_brightness_helper_set_level: failed to set value: %s", error->message);
+    XFPM_DEBUG ("Failed to set value: %s", error->message);
     g_error_free (error);
-    goto out;
+    g_free (command);
+    return FALSE;
   }
-  g_debug ("executed %s; retval: %i", command, exit_status);
-  ret = (exit_status == 0);
 
-out:
   g_free (command);
-  return ret;
+  return TRUE;
 }
 
 static gboolean
@@ -478,104 +408,21 @@ xfpm_brightness_helper_get_switch (XfpmBrightness *brg, gint *brightness_switch)
 static gboolean
 xfpm_brightness_helper_set_switch (XfpmBrightness *brg, gint brightness_switch)
 {
-  gboolean ret;
   GError *error = NULL;
-  gint exit_status = 0;
-  gchar *command = NULL;
+  gint status;
+  gchar *command = g_strdup_printf ("pkexec " SBINDIR "/xfpm-power-backlight-helper --set-brightness-switch %i", brightness_switch);
 
-  command = g_strdup_printf ("pkexec " SBINDIR "/xfpm-power-backlight-helper --set-brightness-switch %i", brightness_switch);
-  ret = g_spawn_command_line_sync (command, NULL, NULL, &exit_status, &error);
-  if ( !ret )
+  XFPM_DEBUG ("Executing command: %s", command);
+  if (!g_spawn_command_line_sync (command, NULL, NULL, &status, &error)
+      || !g_spawn_check_exit_status (status, &error))
   {
-    g_warning ("xfpm_brightness_helper_set_switch: failed to set value: %s", error->message);
+    XFPM_DEBUG ("Failed to set brightness switch value: %s", error->message);
     g_error_free (error);
-    goto out;
+    g_free (command);
+    return FALSE;
   }
-  g_debug ("executed %s; retval: %i", command, exit_status);
-  ret = (exit_status == 0);
 
-out:
   g_free (command);
-  return ret;
-}
-
-static gboolean
-xfpm_brightness_helper_up (XfpmBrightness *brightness, gint32 *new_level)
-{
-  gint32 hw_level;
-  gboolean ret = FALSE;
-  gint32 set_level;
-
-  ret = xfpm_brightness_helper_get_level (brightness, &hw_level);
-
-  if ( !ret )
-    return FALSE;
-
-  if ( hw_level >= brightness->priv->max_level )
-  {
-    *new_level = brightness->priv->max_level;
-    return TRUE;
-  }
-
-  set_level = MIN (xfpm_brightness_inc (brightness, hw_level), brightness->priv->max_level);
-
-  g_warn_if_fail (xfpm_brightness_helper_set_level (brightness, set_level));
-
-  ret = xfpm_brightness_helper_get_level (brightness, new_level);
-
-  if ( !ret )
-  {
-    g_warning ("xfpm_brightness_helper_up failed for %d", set_level);
-    return FALSE;
-  }
-
-  /* Nothing changed in the hardware*/
-  if ( *new_level == hw_level )
-  {
-    g_warning ("xfpm_brightness_helper_up did not change the hw level to %d", set_level);
-    return FALSE;
-  }
-
-  return TRUE;
-}
-
-static gboolean
-xfpm_brightness_helper_down (XfpmBrightness *brightness, gint32 *new_level)
-{
-  gint32 hw_level;
-  gboolean ret;
-  gint32 set_level;
-
-  ret = xfpm_brightness_helper_get_level (brightness, &hw_level);
-
-  if ( !ret )
-    return FALSE;
-
-  if ( hw_level <= brightness->priv->min_level )
-  {
-    *new_level = brightness->priv->min_level;
-    return TRUE;
-  }
-
-  set_level = MAX (xfpm_brightness_dec (brightness, hw_level), brightness->priv->min_level);
-
-  g_warn_if_fail (xfpm_brightness_helper_set_level (brightness, set_level));
-
-  ret = xfpm_brightness_helper_get_level (brightness, new_level);
-
-  if ( !ret )
-  {
-    g_warning ("xfpm_brightness_helper_down failed for %d", set_level);
-    return FALSE;
-  }
-
-  /* Nothing changed in the hardware*/
-  if ( *new_level == hw_level )
-  {
-    g_warning ("xfpm_brightness_helper_down did not change the hw level to %d", set_level);
-    return FALSE;
-  }
-
   return TRUE;
 }
 
@@ -625,15 +472,7 @@ xfpm_brightness_finalize (GObject *object)
   G_OBJECT_CLASS (xfpm_brightness_parent_class)->finalize (object);
 }
 
-XfpmBrightness *
-xfpm_brightness_new (void)
-{
-  XfpmBrightness *brightness = NULL;
-  brightness = g_object_new (XFPM_TYPE_BRIGHTNESS, NULL);
-  return brightness;
-}
-
-gboolean
+static gboolean
 xfpm_brightness_setup (XfpmBrightness *brightness)
 {
   xfpm_brightness_free_data (brightness);
@@ -645,9 +484,9 @@ xfpm_brightness_setup (XfpmBrightness *brightness)
                                      brightness->priv->output,
                                      &brightness->priv->min_level,
                                      &brightness->priv->max_level);
-    g_debug ("Brightness controlled by xrandr, min_level=%d max_level=%d",
-             brightness->priv->min_level,
-             brightness->priv->max_level);
+    XFPM_DEBUG ("Brightness controlled by xrandr, min_level=%d max_level=%d",
+                brightness->priv->min_level,
+                brightness->priv->max_level);
 
     return TRUE;
   }
@@ -656,59 +495,74 @@ xfpm_brightness_setup (XfpmBrightness *brightness)
   {
     if ( xfpm_brightness_setup_helper (brightness) ) {
 #if defined(BACKEND_TYPE_FREEBSD)
-      g_debug ("xrandr not available, brightness controlled by sysctl helper; min_level=%d max_level=%d",
+      XFPM_DEBUG ("xrandr not available, brightness controlled by sysctl helper; min_level=%d max_level=%d",
 #else
-      g_debug ("xrandr not available, brightness controlled by sysfs helper; min_level=%d max_level=%d",
+      XFPM_DEBUG ("xrandr not available, brightness controlled by sysfs helper; min_level=%d max_level=%d",
 #endif
-               brightness->priv->min_level,
-               brightness->priv->max_level);
+                  brightness->priv->min_level,
+                  brightness->priv->max_level);
       return TRUE;
     }
   }
 #endif
-  g_debug ("no brightness controls available");
+  XFPM_DEBUG ("no brightness controls available");
   return FALSE;
 }
 
-gboolean xfpm_brightness_up (XfpmBrightness *brightness, gint32 *new_level)
+XfpmBrightness *
+xfpm_brightness_new (void)
 {
-  gboolean ret = FALSE;
+  XfpmBrightness *brightness = g_object_new (XFPM_TYPE_BRIGHTNESS, NULL);
+  if (!xfpm_brightness_setup (brightness))
+  {
+    g_object_unref (brightness);
+    return NULL;
+  }
 
-  if ( brightness->priv->xrandr_has_hw )
-  {
-    ret = xfpm_brightness_xrand_up (brightness, new_level);
-  }
-#ifdef ENABLE_POLKIT
-  else if ( brightness->priv->helper_has_hw )
-  {
-    ret = xfpm_brightness_helper_up (brightness, new_level);
-  }
-#endif
-  return ret;
+  brightness->priv->hw_min_level = brightness->priv->min_level;
+
+  return brightness;
 }
 
-gboolean xfpm_brightness_down (XfpmBrightness *brightness, gint32 *new_level)
+gint32 xfpm_brightness_get_min_level (XfpmBrightness *brightness)
 {
-  gboolean ret = FALSE;
-
-  if ( brightness->priv->xrandr_has_hw )
-  {
-    ret = xfpm_brightness_xrand_down (brightness, new_level);
-    if ( ret )
-      ret = xfpm_brightness_xrandr_get_level (brightness, brightness->priv->output, new_level);
-  }
-#ifdef ENABLE_POLKIT
-  else if ( brightness->priv->helper_has_hw )
-  {
-    ret = xfpm_brightness_helper_down (brightness, new_level);
-  }
-#endif
-  return ret;
+  return brightness->priv->min_level;
 }
 
-gboolean xfpm_brightness_has_hw (XfpmBrightness *brightness)
+/*
+ * Some laptops (and mostly newer ones with intel graphics) can turn off the
+ * backlight completely. If the user is not careful and sets the brightness
+ * very low using the slider, he might not be able to see the screen contents
+ * anymore. Brightness keys do not work on every laptop, so it's better to use
+ * a safe default minimum level that the user can change via the settings
+ * editor if desired.
+ */
+void
+xfpm_brightness_set_min_level (XfpmBrightness *brightness,
+                               gint32 level)
 {
-  return brightness->priv->xrandr_has_hw || brightness->priv->helper_has_hw;
+  gint32 max_min = brightness->priv->max_level - brightness->priv->step;
+
+  /* -1 = auto, we set the minimum as 10% of delta */
+  if (level == -1)
+  {
+    brightness->priv->min_level = brightness->priv->hw_min_level
+      + MAX (brightness->priv->step, (brightness->priv->max_level - brightness->priv->hw_min_level) / 10);
+    XFPM_DEBUG ("Setting default min brightness (%d) above hardware min (%d)",
+                brightness->priv->min_level, brightness->priv->hw_min_level);
+    return;
+  }
+
+  if (level < brightness->priv->hw_min_level || level > max_min)
+  {
+    XFPM_DEBUG ("Set min brightness (%d) clamped to admissible values [%d, %d]",
+                level, brightness->priv->hw_min_level, max_min);
+    brightness->priv->min_level = CLAMP (level, brightness->priv->hw_min_level, max_min);
+    return;
+  }
+
+  XFPM_DEBUG ("Setting min brightness at %d", level);
+  brightness->priv->min_level = level;
 }
 
 gint32 xfpm_brightness_get_max_level (XfpmBrightness *brightness)
@@ -734,6 +588,13 @@ gboolean xfpm_brightness_set_level (XfpmBrightness *brightness, gint32 level)
 {
   gboolean ret = FALSE;
 
+  if (level < brightness->priv->min_level || level > brightness->priv->max_level)
+  {
+    XFPM_DEBUG ("Set brightness (%d) clamped to admissible values [%d, %d]",
+                level, brightness->priv->min_level, brightness->priv->max_level);
+    level = CLAMP (level, brightness->priv->min_level, brightness->priv->max_level);
+  }
+
   if (brightness->priv->xrandr_has_hw )
     ret = xfpm_brightness_xrandr_set_level (brightness, brightness->priv->output, level);
 #ifdef ENABLE_POLKIT
@@ -744,37 +605,17 @@ gboolean xfpm_brightness_set_level (XfpmBrightness *brightness, gint32 level)
   return ret;
 }
 
-gboolean xfpm_brightness_set_step_count (XfpmBrightness *brightness, guint32 count, gboolean exponential)
+void xfpm_brightness_set_step_count (XfpmBrightness *brightness, guint32 count, gboolean exponential)
 {
-  gboolean ret = FALSE;
+  guint32 delta;
 
-  if ( xfpm_brightness_has_hw (brightness) ) {
-    gint32 delta;
+  if (count < 2)
+    count = 2;
 
-    if ( count < 2 )
-      count = 2;
-    delta = brightness->priv->max_level - brightness->priv->min_level;
-    brightness->priv->use_exp_step = exponential;
-    brightness->priv->step = (delta < (count * 2)) ? 1 : (delta / count);
-    brightness->priv->exp_step = powf (delta, 1.0 / count);
-    ret = TRUE;
-  }
-
-  return ret;
-}
-
-gboolean xfpm_brightness_dim_down (XfpmBrightness *brightness)
-{
-  gboolean ret = FALSE;
-
-  if (brightness->priv->xrandr_has_hw )
-    ret = xfpm_brightness_xrandr_set_level (brightness, brightness->priv->output, brightness->priv->min_level);
-#ifdef ENABLE_POLKIT
-  else if ( brightness->priv->helper_has_hw )
-    ret = xfpm_brightness_helper_set_level (brightness, brightness->priv->min_level);
-#endif
-
-  return ret;
+  delta = brightness->priv->max_level - brightness->priv->hw_min_level;
+  brightness->priv->use_exp_step = exponential;
+  brightness->priv->step = (delta < (count * 2)) ? 1 : (delta / count);
+  brightness->priv->exp_step = powf (delta, 1.0 / count);
 }
 
 gboolean xfpm_brightness_get_switch (XfpmBrightness *brightness, gint *brightness_switch)
