@@ -18,6 +18,10 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "xfpm-dbus.h"
 
 gboolean
@@ -48,7 +52,9 @@ xfpm_dbus_name_has_owner (GDBusConnection *connection, const gchar *name)
 
   if ( error )
   {
-    g_warning("Failed to get name owner: %s\n",error->message);
+    if (! g_error_matches (error, G_DBUS_ERROR, G_DBUS_ERROR_NAME_HAS_NO_OWNER))
+      g_warning("Failed to get name owner: %s\n", error->message);
+
     g_error_free(error);
     return FALSE;
   }
