@@ -765,7 +765,13 @@ on_battery_lid_changed_cb (GtkWidget *w, XfconfChannel *channel)
   {
     g_critical ("Cannot set value for property %s\n", LID_SWITCH_ON_BATTERY_CFG);
   }
-  xfpm_update_logind_handle_lid_switch (channel);
+
+  /* Light Locker Integration */
+  if ( light_locker_settings )
+  {
+    xfpm_update_logind_handle_lid_switch (channel);
+  }
+  /* END Light Locker Integration */
 }
 
 void
@@ -790,7 +796,13 @@ on_ac_lid_changed_cb (GtkWidget *w, XfconfChannel *channel)
   {
     g_critical ("Cannot set value for property %s\n", LID_SWITCH_ON_AC_CFG);
   }
-  xfpm_update_logind_handle_lid_switch (channel);
+
+  /* Light Locker Integration */
+  if ( light_locker_settings )
+  {
+    xfpm_update_logind_handle_lid_switch (channel);
+  }
+  /* END Light Locker Integration */
 }
 
 void
@@ -1907,7 +1919,7 @@ update_device_info_value_for_name (GtkTreeView *view,
   g_return_if_fail (name != NULL);
   /* Value can be NULL */
 
-  DBG ("updating  name %s with value %s", name, value);
+  XFPM_DEBUG ("updating  name %s with value %s", name, value);
 
   iter = find_device_info_name_in_tree (view, name);
   if (iter == NULL)
@@ -1948,8 +1960,6 @@ update_sideview_icon (UpDevice *device,
 
   list_store = GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (sideview)));
 
-  TRACE("entering for %s", object_path);
-
   iter = find_device_in_tree (object_path);
 
   /* quit if device doesn't exist in the sidebar */
@@ -1964,8 +1974,11 @@ update_sideview_icon (UpDevice *device,
                 NULL);
 
 
-  name = get_device_description (upower, device);
-  icon_name = get_device_icon_name (upower, device, FALSE);
+  if (upower != NULL)
+  {
+    name = get_device_description (upower, device);
+    icon_name = get_device_icon_name (upower, device, FALSE);
+  }
 
   pix = gtk_icon_theme_load_icon_for_scale (gtk_icon_theme_get_default (),
                                             icon_name,
@@ -2004,14 +2017,12 @@ update_device_details (UpDevice *device)
   GtkTreeIter *sideview_iter;
   gchar *str;
   guint type = 0, tech = 0;
-  gdouble energy_full_design = -1.0, energy_full = -1.0, energy_empty = -1.0, voltage = -1.0, percent = -1.0;
+  gdouble energy_full_design = -1.0, energy_full = -1.0, energy_empty = -1.0, energy_rate = -1.0, voltage = -1.0, percent = -1.0;
   gboolean p_supply = FALSE;
   gchar *model = NULL, *vendor = NULL, *serial = NULL;
   const gchar *battery_type = NULL;
   const gchar *object_path = up_device_get_object_path(device);
 
-  TRACE("entering for %s", object_path);
-
   sideview_iter = find_device_in_tree (object_path);
 
   /* quit if device doesn't exist in the sidebar */
@@ -2045,6 +2056,7 @@ update_device_details (UpDevice *device)
                 "energy-full-design", &energy_full_design,
                 "energy-full", &energy_full,
                 "energy-empty", &energy_empty,
+                "energy-rate", &energy_rate,
                 "voltage", &voltage,
                 "percentage", &percent,
                 NULL);
@@ -2114,6 +2126,16 @@ update_device_details (UpDevice *device)
       g_free (str);
     }
 
+    if (energy_rate > 0)
+    {
+      /* TRANSLATORS: Unit here is Watt*/
+      str = xfpm_info_get_energy_property (energy_rate, _("W"));
+
+      update_device_info_value_for_name (view, list_store, _("Energy rate"), str);
+
+      g_free (str);
+    }
+
     if (voltage > 0)
     {
       /* TRANSLATORS: Unit here is Volt*/
@@ -2158,8 +2180,6 @@ add_device (UpDevice *device)
   guint index;
   static gboolean first_run = TRUE;
 
-  TRACE("entering for %s", object_path);
-
   /* don't add the same device twice */
   device_iter = find_device_in_tree (object_path);
   if (device_iter)
@@ -2244,8 +2264,6 @@ remove_device (const gchar *object_path)
   gulong signal_id;
   UpDevice *device;
 
-  TRACE("entering for %s", object_path);
-
   iter = find_device_in_tree (object_path);
 
   if (iter == NULL)
@@ -2312,10 +2330,12 @@ settings_create_devices_list (void)
 {
   upower = up_client_new ();
 
-  g_signal_connect (upower, "device-added", G_CALLBACK (device_added_cb), NULL);
-  g_signal_connect (upower, "device-removed", G_CALLBACK (device_removed_cb), NULL);
-
-  add_all_devices ();
+  if (upower != NULL)
+  {
+    g_signal_connect (upower, "device-added", G_CALLBACK (device_added_cb), NULL);
+    g_signal_connect (upower, "device-removed", G_CALLBACK (device_removed_cb), NULL);
+    add_all_devices ();
+  }
 }
 
 static void
@@ -2346,6 +2366,9 @@ settings_quit (GtkWidget *widget, XfconfChannel *channel)
   g_object_unref (channel);
   xfconf_shutdown ();
   gtk_widget_destroy (widget);
+  if (upower != NULL)
+    g_object_unref (upower);
+
   /* initiate the quit action on the application so it terminates */
   g_action_group_activate_action (G_ACTION_GROUP(app), "quit", NULL);
 }
@@ -2385,7 +2408,6 @@ xfpm_settings_dialog_new (XfconfChannel *channel, gboolean auth_suspend,
   GtkWidget *plugged_box;
   GtkWidget *viewport;
   GtkWidget *hbox;
-  GtkWidget *frame;
   GtkWidget *switch_widget;
   GtkWidget *stack;
   GtkStyleContext *context;
@@ -2551,16 +2573,34 @@ xfpm_settings_dialog_new (XfconfChannel *channel, gboolean auth_suspend,
 
   if ( !has_lcd_brightness )
   {
-    frame = GTK_WIDGET (gtk_builder_get_object (xml, "brightness-frame"));
-    gtk_widget_hide (frame);
-    frame = GTK_WIDGET (gtk_builder_get_object (xml, "handle-brightness-keys"));
-    gtk_widget_hide (frame);
-    frame = GTK_WIDGET (gtk_builder_get_object (xml, "handle-brightness-keys-label"));
-    gtk_widget_hide (frame);
-    frame = GTK_WIDGET (gtk_builder_get_object (xml, "brightness-step-count"));
-    gtk_widget_hide (frame);
-    frame = GTK_WIDGET (gtk_builder_get_object (xml, "brightness-exponential"));
-    gtk_widget_hide (frame);
+    GtkWidget *widget = GTK_WIDGET (gtk_builder_get_object (xml, "label4"));
+    gtk_widget_hide (widget);
+    widget = GTK_WIDGET (gtk_builder_get_object (xml, "brightness-level-label"));
+    gtk_widget_hide (widget);
+    widget = GTK_WIDGET (gtk_builder_get_object (xml, "brightness-inactivity-label"));
+    gtk_widget_hide (widget);
+    widget = GTK_WIDGET (gtk_builder_get_object (xml, "brightness-level-vbox"));
+    gtk_widget_hide (widget);
+    widget = GTK_WIDGET (gtk_builder_get_object (xml, "brightness-inactivity-vbox"));
+    gtk_widget_hide (widget);
+    widget = GTK_WIDGET (gtk_builder_get_object (xml, "label6"));
+    gtk_widget_hide (widget);
+    widget = GTK_WIDGET (gtk_builder_get_object (xml, "brightness-level-label1"));
+    gtk_widget_hide (widget);
+    widget = GTK_WIDGET (gtk_builder_get_object (xml, "brightness-inactivity-label1"));
+    gtk_widget_hide (widget);
+    widget = GTK_WIDGET (gtk_builder_get_object (xml, "brightness-level-vbox1"));
+    gtk_widget_hide (widget);
+    widget = GTK_WIDGET (gtk_builder_get_object (xml, "brightness-inactivity-vbox1"));
+    gtk_widget_hide (widget);
+    widget = GTK_WIDGET (gtk_builder_get_object (xml, "handle-brightness-keys"));
+    gtk_widget_hide (widget);
+    widget = GTK_WIDGET (gtk_builder_get_object (xml, "handle-brightness-keys-label"));
+    gtk_widget_hide (widget);
+    widget = GTK_WIDGET (gtk_builder_get_object (xml, "hbox4"));
+    gtk_widget_hide (widget);
+    widget = GTK_WIDGET (gtk_builder_get_object (xml, "brightness-step-count-label"));
+    gtk_widget_hide (widget);
   }
 
   if ( id != 0 )
@@ -2615,14 +2655,14 @@ xfpm_settings_show_device_id (gchar *device_id)
   gtk_widget_show (gtk_notebook_get_nth_page (GTK_NOTEBOOK (nt), devices_page_num));
   gtk_notebook_set_current_page (GTK_NOTEBOOK (nt), devices_page_num);
 
-  DBG("device_id %s", device_id);
+  XFPM_DEBUG ("device_id %s", device_id);
 
   device_iter = find_device_in_tree (device_id);
   if (device_iter)
   {
     GtkTreeSelection *selection;
 
-    DBG("device found");
+    XFPM_DEBUG ("device found");
 
     selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (sideview));
 
