@@ -47,6 +47,7 @@
 #include "xfpm-brightness.h"
 #include "xfpm-debug.h"
 #include "xfpm-icons.h"
+#include "xfpm-enum-types.h"
 
 static void xfpm_backlight_finalize     (GObject *object);
 
@@ -73,7 +74,6 @@ struct XfpmBacklightPrivate
 
   NotifyNotification *n;
 
-  gboolean      has_hw;
   gboolean      on_battery;
 
   gint32          last_level;
@@ -118,7 +118,7 @@ xfpm_backlight_dim_brightness (XfpmBacklight *backlight)
 
     if ( !ret )
     {
-      g_warning ("Unable to get current brightness level");
+      XFPM_DEBUG ("Unable to get current brightness level");
       return;
     }
 
@@ -221,6 +221,17 @@ xfpm_backlight_reset_cb (EggIdletime *idle, XfpmBacklight *backlight)
   }
 }
 
+/**
+ * This callback is responsible for two functionalities in response to a user
+ * pressing the brightness key:
+ *
+ *   - changing screen brightness;
+ *   - displaying a notification with the updated brightness.
+ *
+ * It is possible for these functions to be enabled independently; this would
+ * be desirable if, for example, a user has another tool configured to manage
+ * the backlight brightness which does not provide any visual feedback.
+ */
 static void
 xfpm_backlight_button_pressed_cb (XfpmButton *button, XfpmButtonKey type, XfpmBacklight *backlight)
 {
@@ -230,6 +241,13 @@ xfpm_backlight_button_pressed_cb (XfpmButton *button, XfpmButtonKey type, XfpmBa
   guint    brightness_step_count;
   gboolean brightness_exponential;
 
+  XFPM_DEBUG_ENUM (type, XFPM_TYPE_BUTTON_KEY, "Received button press event");
+
+  /* this check is required; we are notified about keys used by other functions
+   * e.g. keyboard brightness, sleep key. */
+  if ( type != BUTTON_MON_BRIGHTNESS_UP && type != BUTTON_MON_BRIGHTNESS_DOWN )
+    return;
+
   g_object_get (G_OBJECT (backlight->priv->conf),
                 HANDLE_BRIGHTNESS_KEYS, &handle_brightness_keys,
                 SHOW_BRIGHTNESS_POPUP, &show_popup,
@@ -237,23 +255,29 @@ xfpm_backlight_button_pressed_cb (XfpmButton *button, XfpmButtonKey type, XfpmBa
                 BRIGHTNESS_EXPONENTIAL, &brightness_exponential,
                 NULL);
 
-  if ( type != BUTTON_MON_BRIGHTNESS_UP && type != BUTTON_MON_BRIGHTNESS_DOWN )
-    return; /* sanity check, can this ever happen? */
-
   backlight->priv->block = TRUE;
-  if ( !handle_brightness_keys )
-    ret = xfpm_brightness_get_level (backlight->priv->brightness, &level);
-  else
+
+  /* optionally, handle updating the level and setting the screen brightness */
+  if ( handle_brightness_keys )
   {
-    xfpm_brightness_set_step_count(backlight->priv->brightness,
-                                   brightness_step_count,
-                                   brightness_exponential);
+    xfpm_brightness_set_step_count (backlight->priv->brightness,
+                                    brightness_step_count,
+                                    brightness_exponential);
     if ( type == BUTTON_MON_BRIGHTNESS_UP )
-      ret = xfpm_brightness_up (backlight->priv->brightness, &level);
+    {
+      xfpm_brightness_increase (backlight->priv->brightness);
+    }
     else
-      ret = xfpm_brightness_down (backlight->priv->brightness, &level);
+    {
+      xfpm_brightness_decrease (backlight->priv->brightness);
+    }
   }
-  if ( ret && show_popup)
+
+  /* get the current brightness level */
+  ret = xfpm_brightness_get_level (backlight->priv->brightness, &level);
+
+  /* optionally, show the result in a popup (even if it did not change) */
+  if ( ret && show_popup )
     xfpm_backlight_show (backlight, level);
 }
 
@@ -347,7 +371,6 @@ xfpm_backlight_init (XfpmBacklight *backlight)
   backlight->priv = xfpm_backlight_get_instance_private (backlight);
 
   backlight->priv->brightness = xfpm_brightness_new ();
-  backlight->priv->has_hw     = xfpm_brightness_setup (backlight->priv->brightness);
 
   backlight->priv->notify = NULL;
   backlight->priv->idle   = NULL;
@@ -360,14 +383,9 @@ xfpm_backlight_init (XfpmBacklight *backlight)
   backlight->priv->brightness_exponential = FALSE;
   backlight->priv->brightness_switch_initialized = FALSE;
 
-  if ( !backlight->priv->has_hw )
+  if (backlight->priv->brightness != NULL)
   {
-    g_object_unref (backlight->priv->brightness);
-    backlight->priv->brightness = NULL;
-  }
-  else
-  {
-    gboolean ret, handle_keys;
+    gboolean handle_keys;
 
     backlight->priv->idle   = egg_idletime_new ();
     backlight->priv->conf   = xfpm_xfconf_new ();
@@ -381,14 +399,7 @@ xfpm_backlight_init (XfpmBacklight *backlight)
                             XFPM_PROPERTIES_PREFIX BRIGHTNESS_SWITCH, G_TYPE_INT,
                             G_OBJECT (backlight), BRIGHTNESS_SWITCH);
 
-    ret = xfpm_brightness_get_switch (backlight->priv->brightness,
-                      &backlight->priv->brightness_switch);
-
-    if (ret)
-      g_object_set (G_OBJECT (backlight),
-                    BRIGHTNESS_SWITCH,
-                    backlight->priv->brightness_switch,
-                    NULL);
+    xfpm_brightness_get_switch (backlight->priv->brightness, &backlight->priv->brightness_switch);
     backlight->priv->brightness_switch_initialized = TRUE;
 
     /*
@@ -413,9 +424,9 @@ xfpm_backlight_init (XfpmBacklight *backlight)
     }
     else
     {
-      g_warning ("It seems the kernel brightness switch handling value was "
-                 "not restored properly on exit last time, xfce4-power-manager "
-                 "will try to restore it this time.");
+      XFPM_DEBUG ("It seems the kernel brightness switch handling value was "
+                  "not restored properly on exit last time, xfce4-power-manager "
+                  "will try to restore it this time.");
     }
 
       /* check whether to change the brightness switch */
@@ -428,18 +439,18 @@ xfpm_backlight_init (XfpmBacklight *backlight)
             backlight->priv->brightness_switch,
             NULL);
 
-    g_signal_connect (backlight->priv->idle, "alarm-expired",
-                      G_CALLBACK (xfpm_backlight_alarm_timeout_cb), backlight);
-    g_signal_connect (backlight->priv->idle, "reset",
-                      G_CALLBACK(xfpm_backlight_reset_cb), backlight);
-    g_signal_connect (backlight->priv->button, "button-pressed",
-                      G_CALLBACK (xfpm_backlight_button_pressed_cb), backlight);
-    g_signal_connect_swapped (backlight->priv->conf, "notify::" BRIGHTNESS_ON_AC,
-                              G_CALLBACK (xfpm_backlight_brightness_on_ac_settings_changed), backlight);
-    g_signal_connect_swapped (backlight->priv->conf, "notify::" BRIGHTNESS_ON_BATTERY,
-                              G_CALLBACK (xfpm_backlight_brightness_on_battery_settings_changed), backlight);
-    g_signal_connect (backlight->priv->power, "on-battery-changed",
-                      G_CALLBACK (xfpm_backlight_on_battery_changed_cb), backlight);
+    g_signal_connect_object (backlight->priv->idle, "alarm-expired",
+                             G_CALLBACK (xfpm_backlight_alarm_timeout_cb), backlight, 0);
+    g_signal_connect_object (backlight->priv->idle, "reset",
+                             G_CALLBACK(xfpm_backlight_reset_cb), backlight, 0);
+    g_signal_connect_object (backlight->priv->button, "button-pressed",
+                             G_CALLBACK (xfpm_backlight_button_pressed_cb), backlight, 0);
+    g_signal_connect_object (backlight->priv->conf, "notify::" BRIGHTNESS_ON_AC,
+                             G_CALLBACK (xfpm_backlight_brightness_on_ac_settings_changed), backlight, G_CONNECT_SWAPPED);
+    g_signal_connect_object (backlight->priv->conf, "notify::" BRIGHTNESS_ON_BATTERY,
+                             G_CALLBACK (xfpm_backlight_brightness_on_battery_settings_changed), backlight, G_CONNECT_SWAPPED);
+    g_signal_connect_object (backlight->priv->power, "on-battery-changed",
+                             G_CALLBACK (xfpm_backlight_on_battery_changed_cb), backlight, 0);
 
     g_object_get (G_OBJECT (backlight->priv->power),
                   "on-battery", &backlight->priv->on_battery,
@@ -499,11 +510,11 @@ xfpm_backlight_set_property (GObject *object,
       ret = xfpm_brightness_set_switch (backlight->priv->brightness,
                                         backlight->priv->brightness_switch);
       if (!ret)
-          g_warning ("Unable to set the kernel brightness switch parameter to %d.",
-                     backlight->priv->brightness_switch);
+          XFPM_DEBUG ("Unable to set the kernel brightness switch parameter to %d.",
+                      backlight->priv->brightness_switch);
       else
-          g_message ("Set kernel brightness switch to %d",
-                     backlight->priv->brightness_switch);
+          XFPM_DEBUG ("Set kernel brightness switch to %d",
+                      backlight->priv->brightness_switch);
 
       break;
     case PROP_BRIGHTNESS_SWITCH_SAVE:
@@ -543,11 +554,11 @@ xfpm_backlight_finalize (GObject *object)
     if (ret)
     {
       backlight->priv->brightness_switch = backlight->priv->brightness_switch_save;
-      g_message ("Restored brightness switch value to: %d", backlight->priv->brightness_switch);
+      XFPM_DEBUG ("Restored brightness switch value to: %d", backlight->priv->brightness_switch);
     }
     else
-      g_warning ("Unable to restore the kernel brightness switch parameter to its original value, "
-                 "still resetting the saved value.");
+      XFPM_DEBUG ("Unable to restore the kernel brightness switch parameter to its original value, "
+                  "still resetting the saved value.");
     }
     g_object_unref (backlight->priv->conf);
   }
@@ -570,12 +581,12 @@ xfpm_backlight_finalize (GObject *object)
 XfpmBacklight *
 xfpm_backlight_new (void)
 {
-  XfpmBacklight *backlight = NULL;
-  backlight = g_object_new (XFPM_TYPE_BACKLIGHT, NULL);
+  XfpmBacklight *backlight = g_object_new (XFPM_TYPE_BACKLIGHT, NULL);
+  if (backlight->priv->brightness == NULL)
+  {
+    g_object_unref (backlight);
+    return NULL;
+  }
+
   return backlight;
 }
-
-gboolean xfpm_backlight_has_hw (XfpmBacklight *backlight)
-{
-  return backlight->priv->has_hw;
-}
