@@ -167,15 +167,12 @@ xfpm_button_xevent_key (XfpmButton *button, guint keysym , XfpmButtonKey key)
 
   if ( keycode == 0 )
   {
-    g_warning ("could not map keysym %x to keycode\n", keysym);
+    XFPM_DEBUG ("could not map keysym %x to keycode\n", keysym);
     return FALSE;
   }
 
   if ( !xfpm_button_grab_keystring(button, keycode))
-  {
-    g_warning ("Failed to grab %i\n", keycode);
     return FALSE;
-  }
 
   XFPM_DEBUG_ENUM (key, XFPM_TYPE_BUTTON_KEY, "Grabbed key %li ", (long int) keycode);
 
