@@ -36,6 +36,7 @@
 #include "xfpm-button.h"
 #include "xfpm-notify.h"
 #include "xfpm-power.h"
+#include "xfpm-debug.h"
 
 static void xfpm_kbd_backlight_finalize     (GObject *object);
 
@@ -99,7 +100,7 @@ xfpm_kbd_backlight_init_max_level (XfpmKbdBacklight *backlight)
 
   if ( error )
   {
-    g_warning ("Failed to get keyboard max brightness level : %s", error->message);
+    XFPM_DEBUG ("Failed to get keyboard max brightness level : %s", error->message);
     g_error_free (error);
   }
 }
@@ -306,11 +307,11 @@ xfpm_kbd_backlight_init (XfpmKbdBacklight *backlight)
   backlight->priv->button = xfpm_button_new ();
   backlight->priv->notify = xfpm_notify_new ();
 
-  g_signal_connect (backlight->priv->button, "button-pressed",
-                    G_CALLBACK (xfpm_kbd_backlight_button_pressed_cb), backlight);
+  g_signal_connect_object (backlight->priv->button, "button-pressed",
+                           G_CALLBACK (xfpm_kbd_backlight_button_pressed_cb), backlight, 0);
 
-  g_signal_connect (backlight->priv->power, "on-battery-changed",
-                    G_CALLBACK (xfpm_kbd_backlight_on_battery_changed_cb), backlight);
+  g_signal_connect_object (backlight->priv->power, "on-battery-changed",
+                           G_CALLBACK (xfpm_kbd_backlight_on_battery_changed_cb), backlight, 0);
 
   g_object_get (G_OBJECT (backlight->priv->power),
                 "on-battery", &backlight->priv->on_battery,
