@@ -187,7 +187,8 @@ xfpm_manager_finalize (GObject *object)
 
   g_object_unref (manager->priv->dpms);
 
-  g_object_unref (manager->priv->backlight);
+  if (manager->priv->backlight != NULL)
+    g_object_unref (manager->priv->backlight);
 
   g_object_unref (manager->priv->kbd_backlight);
 
@@ -530,12 +531,10 @@ xfpm_manager_set_idle_alarm_on_ac (XfpmManager *manager)
                 ON_AC_INACTIVITY_TIMEOUT, &on_ac,
                 NULL);
 
-#ifdef DEBUG
   if ( on_ac == 14 )
-    TRACE ("setting inactivity sleep timeout on ac to never");
+    XFPM_DEBUG ("setting inactivity sleep timeout on ac to never");
   else
-    TRACE ("setting inactivity sleep timeout on ac to %d", on_ac);
-#endif
+    XFPM_DEBUG ("setting inactivity sleep timeout on ac to %d", on_ac);
 
   if ( on_ac == 14 )
   {
@@ -556,12 +555,10 @@ xfpm_manager_set_idle_alarm_on_battery (XfpmManager *manager)
                 ON_BATTERY_INACTIVITY_TIMEOUT, &on_battery,
                 NULL);
 
-#ifdef DEBUG
   if ( on_battery == 14 )
-    TRACE ("setting inactivity sleep timeout on battery to never");
+    XFPM_DEBUG ("setting inactivity sleep timeout on battery to never");
   else
-    TRACE ("setting inactivity sleep timeout on battery to %d", on_battery);
-#endif
+    XFPM_DEBUG ("setting inactivity sleep timeout on battery to %d", on_battery);
 
   if ( on_battery == 14 )
   {
@@ -753,7 +750,7 @@ xfpm_manager_show_tray_icon (XfpmManager *manager)
 
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   manager->priv->adapter_icon = gtk_status_icon_new ();
-  manager->priv->power_button = power_manager_button_new ();
+  manager->priv->power_button = g_object_ref_sink (power_manager_button_new ());
 G_GNUC_END_IGNORE_DEPRECATIONS
 
   XFPM_DEBUG ("Showing tray icon");
@@ -835,8 +832,8 @@ xfpm_manager_new (GDBusConnection *bus, const gchar *client_id)
   }
   else
   {
-    g_signal_connect_swapped (manager->priv->client, "quit",
-                              G_CALLBACK (xfpm_manager_quit), manager);
+    g_signal_connect_object (manager->priv->client, "quit",
+                             G_CALLBACK (xfpm_manager_quit), manager, G_CONNECT_SWAPPED);
   }
 
   xfpm_manager_dbus_class_init (XFPM_MANAGER_GET_CLASS (manager));
@@ -879,27 +876,27 @@ void xfpm_manager_start (XfpmManager *manager)
     g_clear_error (&error);
   }
 
-  g_signal_connect (manager->priv->idle, "alarm-expired",
-                    G_CALLBACK (xfpm_manager_alarm_timeout_cb), manager);
-  g_signal_connect_swapped (manager->priv->conf, "notify::" ON_AC_INACTIVITY_TIMEOUT,
-                            G_CALLBACK (xfpm_manager_set_idle_alarm_on_ac), manager);
-  g_signal_connect_swapped (manager->priv->conf, "notify::" ON_BATTERY_INACTIVITY_TIMEOUT,
-                            G_CALLBACK (xfpm_manager_set_idle_alarm_on_battery), manager);
-  g_signal_connect_swapped (manager->priv->conf, "notify::" LOGIND_HANDLE_POWER_KEY,
-                            G_CALLBACK (xfpm_manager_systemd_events_changed), manager);
-  g_signal_connect_swapped (manager->priv->conf, "notify::" LOGIND_HANDLE_SUSPEND_KEY,
-                            G_CALLBACK (xfpm_manager_systemd_events_changed), manager);
-  g_signal_connect_swapped (manager->priv->conf, "notify::" LOGIND_HANDLE_HIBERNATE_KEY,
-                            G_CALLBACK (xfpm_manager_systemd_events_changed), manager);
-  g_signal_connect_swapped (manager->priv->conf, "notify::" LOGIND_HANDLE_LID_SWITCH,
-                            G_CALLBACK (xfpm_manager_systemd_events_changed), manager);
+  g_signal_connect_object (manager->priv->idle, "alarm-expired",
+                           G_CALLBACK (xfpm_manager_alarm_timeout_cb), manager, 0);
+  g_signal_connect_object (manager->priv->conf, "notify::" ON_AC_INACTIVITY_TIMEOUT,
+                           G_CALLBACK (xfpm_manager_set_idle_alarm_on_ac), manager, G_CONNECT_SWAPPED);
+  g_signal_connect_object (manager->priv->conf, "notify::" ON_BATTERY_INACTIVITY_TIMEOUT,
+                           G_CALLBACK (xfpm_manager_set_idle_alarm_on_battery), manager, G_CONNECT_SWAPPED);
+  g_signal_connect_object (manager->priv->conf, "notify::" LOGIND_HANDLE_POWER_KEY,
+                           G_CALLBACK (xfpm_manager_systemd_events_changed), manager, G_CONNECT_SWAPPED);
+  g_signal_connect_object (manager->priv->conf, "notify::" LOGIND_HANDLE_SUSPEND_KEY,
+                           G_CALLBACK (xfpm_manager_systemd_events_changed), manager, G_CONNECT_SWAPPED);
+  g_signal_connect_object (manager->priv->conf, "notify::" LOGIND_HANDLE_HIBERNATE_KEY,
+                           G_CALLBACK (xfpm_manager_systemd_events_changed), manager, G_CONNECT_SWAPPED);
+  g_signal_connect_object (manager->priv->conf, "notify::" LOGIND_HANDLE_LID_SWITCH,
+                           G_CALLBACK (xfpm_manager_systemd_events_changed), manager, G_CONNECT_SWAPPED);
 
   xfpm_manager_set_idle_alarm (manager);
 
-  g_signal_connect (manager->priv->inhibit, "has-inhibit-changed",
-                    G_CALLBACK (xfpm_manager_inhibit_changed_cb), manager);
-  g_signal_connect (manager->priv->monitor, "system-bus-connection-changed",
-                    G_CALLBACK (xfpm_manager_system_bus_connection_changed_cb), manager);
+  g_signal_connect_object (manager->priv->inhibit, "has-inhibit-changed",
+                           G_CALLBACK (xfpm_manager_inhibit_changed_cb), manager, 0);
+  g_signal_connect_object (manager->priv->monitor, "system-bus-connection-changed",
+                           G_CALLBACK (xfpm_manager_system_bus_connection_changed_cb), manager, 0);
 
   manager->priv->backlight = xfpm_backlight_new ();
 
@@ -907,26 +904,26 @@ void xfpm_manager_start (XfpmManager *manager)
 
   manager->priv->dpms = xfpm_dpms_new ();
 
-  g_signal_connect (manager->priv->button, "button_pressed",
-                    G_CALLBACK (xfpm_manager_button_pressed_cb), manager);
+  g_signal_connect_object (manager->priv->button, "button_pressed",
+                           G_CALLBACK (xfpm_manager_button_pressed_cb), manager, 0);
 
-  g_signal_connect (manager->priv->power, "lid-changed",
-                    G_CALLBACK (xfpm_manager_lid_changed_cb), manager);
+  g_signal_connect_object (manager->priv->power, "lid-changed",
+                           G_CALLBACK (xfpm_manager_lid_changed_cb), manager, 0);
 
-  g_signal_connect (manager->priv->power, "on-battery-changed",
-                    G_CALLBACK (xfpm_manager_on_battery_changed_cb), manager);
+  g_signal_connect_object (manager->priv->power, "on-battery-changed",
+                           G_CALLBACK (xfpm_manager_on_battery_changed_cb), manager, 0);
 
-  g_signal_connect_swapped (manager->priv->power, "waking-up",
-                            G_CALLBACK (xfpm_manager_reset_sleep_timer), manager);
+  g_signal_connect_object (manager->priv->power, "waking-up",
+                           G_CALLBACK (xfpm_manager_reset_sleep_timer), manager, G_CONNECT_SWAPPED);
 
-  g_signal_connect_swapped (manager->priv->power, "sleeping",
-                            G_CALLBACK (xfpm_manager_reset_sleep_timer), manager);
+  g_signal_connect_object (manager->priv->power, "sleeping",
+                           G_CALLBACK (xfpm_manager_reset_sleep_timer), manager, G_CONNECT_SWAPPED);
 
-  g_signal_connect_swapped (manager->priv->power, "ask-shutdown",
-                            G_CALLBACK (xfpm_manager_ask_shutdown), manager);
+  g_signal_connect_object (manager->priv->power, "ask-shutdown",
+                           G_CALLBACK (xfpm_manager_ask_shutdown), manager, G_CONNECT_SWAPPED);
 
-  g_signal_connect_swapped (manager->priv->power, "shutdown",
-                            G_CALLBACK (xfpm_manager_shutdown), manager);
+  g_signal_connect_object (manager->priv->power, "shutdown",
+                           G_CALLBACK (xfpm_manager_shutdown), manager, G_CONNECT_SWAPPED);
 
   xfconf_g_property_bind (xfpm_xfconf_get_channel (manager->priv->conf),
                                                    XFPM_PROPERTIES_PREFIX SHOW_TRAY_ICON_CFG,
@@ -958,7 +955,7 @@ GHashTable *xfpm_manager_get_config (XfpmManager *manager)
   gboolean has_power_button = FALSE;
   gboolean has_battery_button = FALSE;
   gboolean has_battery = TRUE;
-  gboolean has_lcd_brightness = TRUE;
+  gboolean has_lcd_brightness = manager->priv->backlight != NULL;
   gboolean can_shutdown = TRUE;
   gboolean has_lid = FALSE;
 
@@ -986,8 +983,6 @@ GHashTable *xfpm_manager_get_config (XfpmManager *manager)
                 NULL);
 
   has_battery = xfpm_power_has_battery (manager->priv->power);
-  has_lcd_brightness = xfpm_backlight_has_hw (manager->priv->backlight);
-
   mapped_buttons = xfpm_button_get_mapped (manager->priv->button);
 
   if ( mapped_buttons & SLEEP_KEY )
@@ -1055,22 +1050,22 @@ xfpm_manager_dbus_init (XfpmManager *manager)
                                     "/org/xfce/PowerManager",
                                     NULL);
 
-  g_signal_connect_swapped (manager_dbus,
-                            "handle-quit",
-                            G_CALLBACK (xfpm_manager_dbus_quit),
-                            manager);
-  g_signal_connect_swapped (manager_dbus,
-                            "handle-restart",
-                            G_CALLBACK (xfpm_manager_dbus_restart),
-                            manager);
-  g_signal_connect_swapped (manager_dbus,
-                            "handle-get-config",
-                            G_CALLBACK (xfpm_manager_dbus_get_config),
-                            manager);
-  g_signal_connect_swapped (manager_dbus,
-                            "handle-get-info",
-                            G_CALLBACK (xfpm_manager_dbus_get_info),
-                            manager);
+  g_signal_connect_object (manager_dbus,
+                           "handle-quit",
+                           G_CALLBACK (xfpm_manager_dbus_quit),
+                           manager, G_CONNECT_SWAPPED);
+  g_signal_connect_object (manager_dbus,
+                           "handle-restart",
+                           G_CALLBACK (xfpm_manager_dbus_restart),
+                           manager, G_CONNECT_SWAPPED);
+  g_signal_connect_object (manager_dbus,
+                           "handle-get-config",
+                           G_CALLBACK (xfpm_manager_dbus_get_config),
+                           manager, G_CONNECT_SWAPPED);
+  g_signal_connect_object (manager_dbus,
+                           "handle-get-info",
+                           G_CALLBACK (xfpm_manager_dbus_get_info),
+                           manager, G_CONNECT_SWAPPED);
 }
 
 static gboolean
