@@ -100,14 +100,11 @@ struct XfpmPowerPrivate
   gboolean          critical_action_done;
 
   XfpmDpms         *dpms;
+  XfceScreensaver  *screensaver;
   gboolean          presentation_mode;
+  gboolean          inhibited;
   gint              on_ac_blank;
   gint              on_battery_blank;
-  EggIdletime      *idletime;
-
-  gboolean          inhibited;
-  gboolean          screensaver_inhibited;
-  XfceScreensaver  *screensaver;
 
   XfpmNotify       *notify;
 #ifdef ENABLE_POLKIT
@@ -243,7 +240,7 @@ xfpm_power_check_power (XfpmPower *power, gboolean on_battery)
   if (on_battery != power->priv->on_battery )
   {
     GList *list;
-    guint len, i;
+
     g_signal_emit (G_OBJECT (power), signals [ON_BATTERY_CHANGED], 0, on_battery);
 
     xfpm_dpms_set_on_battery (power->priv->dpms, on_battery);
@@ -253,14 +250,14 @@ xfpm_power_check_power (XfpmPower *power, gboolean on_battery)
 
     power->priv->on_battery = on_battery;
     list = g_hash_table_get_values (power->priv->hash);
-    len = g_list_length (list);
-    for ( i = 0; i < len; i++)
+    for (GList *lp = list; lp != NULL; lp = lp->next)
     {
-      g_object_set (G_OBJECT (g_list_nth_data (list, i)),
+      g_object_set (G_OBJECT (lp->data),
                     "ac-online", !on_battery,
                     NULL);
       xfpm_update_blank_time (power);
     }
+    g_list_free (list);
   }
 }
 
@@ -293,9 +290,9 @@ xfpm_power_check_lid (XfpmPower *power, gboolean present, gboolean closed)
 static void
 xfpm_power_get_properties (XfpmPower *power)
 {
-  gboolean on_battery;
-  gboolean lid_is_closed;
-  gboolean lid_is_present;
+  gboolean on_battery = FALSE;
+  gboolean lid_is_closed = FALSE;
+  gboolean lid_is_present = FALSE;
 
   if (power->priv->systemd != NULL)
   {
@@ -324,11 +321,12 @@ xfpm_power_get_properties (XfpmPower *power)
     }
   }
 
-  g_object_get (power->priv->upower,
-                "on-battery", &on_battery,
-                "lid-is-closed", &lid_is_closed,
-                "lid-is-present", &lid_is_present,
-                NULL);
+  if (power->priv->upower != NULL)
+    g_object_get (power->priv->upower,
+                  "on-battery", &on_battery,
+                  "lid-is-closed", &lid_is_closed,
+                  "lid-is-present", &lid_is_present,
+                  NULL);
   xfpm_power_check_lid (power, lid_is_present, lid_is_closed);
   xfpm_power_check_power (power, on_battery);
 }
@@ -336,23 +334,6 @@ xfpm_power_get_properties (XfpmPower *power)
 static void
 xfpm_power_report_error (XfpmPower *power, const gchar *error, const gchar *icon_name)
 {
-  GtkStatusIcon *battery = NULL;
-  guint i, len;
-  GList *list;
-
-  list = g_hash_table_get_values (power->priv->hash);
-  len = g_list_length (list);
-
-  for ( i = 0; i < len; i++)
-  {
-    UpDeviceKind type;
-    battery = g_list_nth_data (list, i);
-    type = xfpm_battery_get_device_type (XFPM_BATTERY (battery));
-    if ( type == UP_DEVICE_KIND_BATTERY ||
-         type == UP_DEVICE_KIND_UPS )
-      break;
-  }
-
   xfpm_notify_show_notification (power->priv->notify,
                                  _("Power Manager"),
                                  error,
@@ -369,7 +350,7 @@ xfpm_power_sleep (XfpmPower *power, const gchar *sleep_time, gboolean force)
   gboolean network_manager_sleep;
 #endif
   XfpmBrightness *brightness;
-  gint32 brightness_level;
+  gint32 brightness_level = 0;
 
   if ( power->priv->inhibited && force == FALSE)
   {
@@ -393,8 +374,8 @@ xfpm_power_sleep (XfpmPower *power, const gchar *sleep_time, gboolean force)
   g_signal_emit (G_OBJECT (power), signals [SLEEPING], 0);
     /* Get the current brightness level so we can use it after we suspend */
   brightness = xfpm_brightness_new();
-  xfpm_brightness_setup (brightness);
-  xfpm_brightness_get_level (brightness, &brightness_level);
+  if (brightness != NULL)
+    xfpm_brightness_get_level (brightness, &brightness_level);
 
 #ifdef WITH_NETWORK_MANAGER
   g_object_get (G_OBJECT (power->priv->conf),
@@ -420,7 +401,7 @@ xfpm_power_sleep (XfpmPower *power, const gchar *sleep_time, gboolean force)
       g_usleep (2000000);
     }
 #endif
-    if (!xfce_screensaver_lock (power->priv->screensaver))
+    if (!xfce_screensaver_lock (power->priv->screensaver) && !force)
     {
       GtkWidget *dialog;
       gboolean ret;
@@ -438,9 +419,13 @@ xfpm_power_sleep (XfpmPower *power, const gchar *sleep_time, gboolean force)
       gtk_widget_destroy (dialog);
 
       if ( !ret || ret == GTK_RESPONSE_NO)
+      {
+        if (brightness != NULL)
+          g_object_unref (brightness);
         return;
       }
     }
+  }
 
     /* This is fun, here's the order of operations:
      * - if the Logind is running then use it
@@ -495,7 +480,11 @@ xfpm_power_sleep (XfpmPower *power, const gchar *sleep_time, gboolean force)
     /* Check/update any changes while we slept */
   xfpm_power_get_properties (power);
     /* Restore the brightness level from before we suspended */
-  xfpm_brightness_set_level (brightness, brightness_level);
+  if (brightness != NULL)
+  {
+    xfpm_brightness_set_level (brightness, brightness_level);
+    g_object_unref (brightness);
+  }
 
 #ifdef WITH_NETWORK_MANAGER
   if ( network_manager_sleep )
@@ -533,19 +522,16 @@ static XfpmBatteryCharge
 xfpm_power_get_current_charge_state (XfpmPower *power)
 {
   GList *list;
-  guint len, i;
-  gboolean power_supply;
   XfpmBatteryCharge max_charge_status = XFPM_BATTERY_CHARGE_UNKNOWN;
 
   list = g_hash_table_get_values (power->priv->hash);
-  len = g_list_length (list);
-
-  for ( i = 0; i < len; i++ )
+  for (GList *lp = list; lp != NULL; lp = lp->next)
   {
     XfpmBatteryCharge battery_charge;
     UpDeviceKind type;
+    gboolean power_supply;
 
-    g_object_get (G_OBJECT (g_list_nth_data (list, i)),
+    g_object_get (G_OBJECT (lp->data),
                     "charge-status", &battery_charge,
                     "device-type", &type,
                     "ac-online", &power_supply,
@@ -557,6 +543,7 @@ xfpm_power_get_current_charge_state (XfpmPower *power)
 
     max_charge_status = MAX (max_charge_status, battery_charge);
   }
+  g_list_free (list);
 
   return max_charge_status;
 }
@@ -689,8 +676,8 @@ xfpm_power_show_critical_action_gtk (XfpmPower *power)
     hibernate = gtk_button_new_with_label (_("Hibernate"));
     gtk_dialog_add_action_widget (GTK_DIALOG (dialog), hibernate, GTK_RESPONSE_NONE);
 
-    g_signal_connect_swapped (hibernate, "clicked",
-                              G_CALLBACK (xfpm_power_hibernate_clicked), power);
+    g_signal_connect_object (hibernate, "clicked",
+                             G_CALLBACK (xfpm_power_hibernate_clicked), power, G_CONNECT_SWAPPED);
   }
 
   if ( power->priv->can_suspend && power->priv->auth_suspend )
@@ -700,9 +687,9 @@ xfpm_power_show_critical_action_gtk (XfpmPower *power)
     suspend = gtk_button_new_with_label (_("Suspend"));
     gtk_dialog_add_action_widget (GTK_DIALOG (dialog), suspend, GTK_RESPONSE_NONE);
 
-    g_signal_connect_swapped (suspend, "clicked",
-                              G_CALLBACK (xfpm_power_suspend_clicked), power);
-    }
+    g_signal_connect_object (suspend, "clicked",
+                             G_CALLBACK (xfpm_power_suspend_clicked), power, G_CONNECT_SWAPPED);
+  }
 
   if ( can_shutdown )
   {
@@ -711,18 +698,18 @@ xfpm_power_show_critical_action_gtk (XfpmPower *power)
     shutdown = gtk_button_new_with_label (_("Shutdown"));
     gtk_dialog_add_action_widget (GTK_DIALOG (dialog), shutdown, GTK_RESPONSE_NONE);
 
-    g_signal_connect_swapped (shutdown, "clicked",
-                              G_CALLBACK (xfpm_power_shutdown_clicked), power);
+    g_signal_connect_object (shutdown, "clicked",
+                             G_CALLBACK (xfpm_power_shutdown_clicked), power, G_CONNECT_SWAPPED);
   }
 
   cancel = gtk_button_new_with_label (_("_Cancel"));
   gtk_dialog_add_action_widget (GTK_DIALOG (dialog), cancel, GTK_RESPONSE_NONE);
 
-  g_signal_connect_swapped (cancel, "clicked",
-                            G_CALLBACK (xfpm_power_close_critical_dialog), power);
+  g_signal_connect_object (cancel, "clicked",
+                           G_CALLBACK (xfpm_power_close_critical_dialog), power, G_CONNECT_SWAPPED);
 
-  g_signal_connect_swapped (dialog, "destroy",
-                            G_CALLBACK (xfpm_power_close_critical_dialog), power);
+  g_signal_connect_object (dialog, "destroy",
+                           G_CALLBACK (xfpm_power_close_critical_dialog), power, G_CONNECT_SWAPPED);
   if ( power->priv->dialog )
   {
     gtk_widget_destroy (power->priv->dialog);
@@ -903,8 +890,8 @@ xfpm_power_add_device (UpDevice *device, XfpmPower *power)
                                  device_type);
     g_hash_table_insert (power->priv->hash, g_strdup (object_path), battery);
 
-    g_signal_connect (battery, "battery-charge-changed",
-                      G_CALLBACK (xfpm_power_battery_charge_changed_cb), power);
+    g_signal_connect_object (battery, "battery-charge-changed",
+                             G_CALLBACK (xfpm_power_battery_charge_changed_cb), power, 0);
   }
 }
 
@@ -942,43 +929,23 @@ xfpm_power_remove_device (XfpmPower *power, const gchar *object_path)
 static void
 xfpm_power_inhibit_changed_cb (XfpmInhibit *inhibit, gboolean is_inhibit, XfpmPower *power)
 {
-  if (power->priv->inhibited != is_inhibit)
-  {
-    power->priv->inhibited = is_inhibit;
+  /* no change, exit */
+  if (power->priv->inhibited == is_inhibit)
+    return;
 
-    XFPM_DEBUG ("is_inhibit %s, screensaver_inhibited %s, presentation_mode %s",
-                power->priv->inhibited ? "TRUE" : "FALSE",
-                power->priv->screensaver_inhibited ? "TRUE" : "FALSE",
-                power->priv->presentation_mode ? "TRUE" : "FALSE");
+  power->priv->inhibited = is_inhibit;
 
-    xfpm_dpms_inhibit (power->priv->dpms, is_inhibit);
+  XFPM_DEBUG ("inhibited %s, presentation_mode %s",
+              power->priv->inhibited ? "TRUE" : "FALSE",
+              power->priv->presentation_mode ? "TRUE" : "FALSE");
 
-    /* If we are inhibited make sure we inhibit the screensaver too */
-    if (is_inhibit)
-    {
-      if (!power->priv->screensaver_inhibited)
-      {
-        xfce_screensaver_inhibit (power->priv->screensaver, TRUE);
-        power->priv->screensaver_inhibited = TRUE;
-      }
-    }
-    else
-    {
-      /* Or make sure we remove the screensaver inhibit */
-      if (power->priv->screensaver_inhibited && !power->priv->presentation_mode)
-      {
-        xfce_screensaver_inhibit (power->priv->screensaver, FALSE);
-        power->priv->screensaver_inhibited = FALSE;
-      }
-    }
-  }
-
-  XFPM_DEBUG ("is_inhibit %s, screensaver_inhibited %s, presentation_mode %s",
-  power->priv->inhibited ? "TRUE" : "FALSE",
-  power->priv->screensaver_inhibited ? "TRUE" : "FALSE",
-  power->priv->presentation_mode ? "TRUE" : "FALSE");
+  /* either inhibition already occurred, or we don't want to remove it yet */
+  if (power->priv->presentation_mode)
+    return;
 
+  xfce_screensaver_inhibit (power->priv->screensaver, is_inhibit);
   xfpm_update_blank_time (power);
+  xfpm_dpms_inhibit (power->priv->dpms, is_inhibit);
 }
 
 static void
@@ -1209,12 +1176,12 @@ xfpm_power_init (XfpmPower *power)
 
 #ifdef ENABLE_POLKIT
   power->priv->polkit  = xfpm_polkit_get ();
-  g_signal_connect_swapped (power->priv->polkit, "auth-changed",
-                            G_CALLBACK (xfpm_power_polkit_auth_changed_cb), power);
+  g_signal_connect_object (power->priv->polkit, "auth-changed",
+                           G_CALLBACK (xfpm_power_polkit_auth_changed_cb), power, G_CONNECT_SWAPPED);
 #endif
 
-  g_signal_connect (power->priv->inhibit, "has-inhibit-changed",
-                    G_CALLBACK (xfpm_power_inhibit_changed_cb), power);
+  g_signal_connect_object (power->priv->inhibit, "has-inhibit-changed",
+                           G_CALLBACK (xfpm_power_inhibit_changed_cb), power, 0);
 
   power->priv->bus = g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, &error);
 
@@ -1225,11 +1192,14 @@ xfpm_power_init (XfpmPower *power)
     goto out;
   }
 
-  g_signal_connect (power->priv->upower, "device-added", G_CALLBACK (xfpm_power_device_added_cb), power);
-  g_signal_connect (power->priv->upower, "device-removed", G_CALLBACK (xfpm_power_device_removed_cb), power);
-  g_signal_connect (power->priv->upower, "notify", G_CALLBACK (xfpm_power_changed_cb), power);
+  if (power->priv->upower != NULL)
+  {
+    g_signal_connect_object (power->priv->upower, "device-added", G_CALLBACK (xfpm_power_device_added_cb), power, 0);
+    g_signal_connect_object (power->priv->upower, "device-removed", G_CALLBACK (xfpm_power_device_removed_cb), power, 0);
+    g_signal_connect_object (power->priv->upower, "notify", G_CALLBACK (xfpm_power_changed_cb), power, 0);
+    xfpm_power_get_power_devices (power);
+  }
 
-  xfpm_power_get_power_devices (power);
   xfpm_power_get_properties (power);
 #ifdef ENABLE_POLKIT
   xfpm_power_check_polkit_auth (power);
@@ -1333,6 +1303,8 @@ xfpm_power_finalize (GObject *object)
   g_object_unref (power->priv->notify);
   g_object_unref (power->priv->conf);
   g_object_unref (power->priv->screensaver);
+  if (power->priv->upower != NULL)
+    g_object_unref (power->priv->upower);
 
   if ( power->priv->systemd != NULL )
     g_object_unref (power->priv->systemd);
@@ -1402,20 +1374,14 @@ void xfpm_power_hibernate (XfpmPower *power, gboolean force)
 
 gboolean xfpm_power_has_battery (XfpmPower *power)
 {
-  GtkStatusIcon *battery = NULL;
-  guint i, len;
   GList *list;
-
   gboolean ret = FALSE;
 
   list = g_hash_table_get_values (power->priv->hash);
-  len = g_list_length (list);
-
-  for ( i = 0; i < len; i++)
+  for (GList *lp = list; lp != NULL; lp = lp->next)
   {
     UpDeviceKind type;
-    battery = g_list_nth_data (list, i);
-    type = xfpm_battery_get_device_type (XFPM_BATTERY (battery));
+    type = xfpm_battery_get_device_type (XFPM_BATTERY (lp->data));
     if ( type == UP_DEVICE_KIND_BATTERY ||
          type == UP_DEVICE_KIND_UPS )
     {
@@ -1423,6 +1389,7 @@ gboolean xfpm_power_has_battery (XfpmPower *power)
       break;
     }
   }
+  g_list_free (list);
 
   return ret;
 }
@@ -1454,53 +1421,23 @@ xfpm_update_blank_time (XfpmPower *power)
 static void
 xfpm_power_change_presentation_mode (XfpmPower *power, gboolean presentation_mode)
 {
-    /* no change, exit */
+  /* no change, exit */
   if (power->priv->presentation_mode == presentation_mode)
     return;
 
   power->priv->presentation_mode = presentation_mode;
 
-  /* presentation mode inhibits dpms */
-  xfpm_dpms_inhibit (power->priv->dpms, presentation_mode);
+  XFPM_DEBUG ("inhibited %s, presentation_mode %s",
+              power->priv->inhibited ? "TRUE" : "FALSE",
+              power->priv->presentation_mode ? "TRUE" : "FALSE");
 
-  XFPM_DEBUG ("is_inhibit %s, screensaver_inhibited %s, presentation_mode %s",
-  power->priv->inhibited ? "TRUE" : "FALSE",
-  power->priv->screensaver_inhibited ? "TRUE" : "FALSE",
-  power->priv->presentation_mode ? "TRUE" : "FALSE");
-
-  if (presentation_mode)
-  {
-  /* presentation mode inhibits the screensaver */
-    if (!power->priv->screensaver_inhibited)
-    {
-      xfce_screensaver_inhibit (power->priv->screensaver, TRUE);
-      power->priv->screensaver_inhibited = TRUE;
-    }
-  }
-  else
-  {
-    EggIdletime *idletime;
-
-    /* make sure we remove the screensaver inhibit */
-    if (power->priv->screensaver_inhibited && !power->priv->inhibited)
-    {
-      xfce_screensaver_inhibit (power->priv->screensaver, FALSE);
-      power->priv->screensaver_inhibited = FALSE;
-    }
-
-    /* reset the timers */
-    idletime = egg_idletime_new ();
-    egg_idletime_alarm_reset_all (idletime);
-
-    g_object_unref (idletime);
-  }
-
-  XFPM_DEBUG ("is_inhibit %s, screensaver_inhibited %s, presentation_mode %s",
-  power->priv->inhibited ? "TRUE" : "FALSE",
-  power->priv->screensaver_inhibited ? "TRUE" : "FALSE",
-  power->priv->presentation_mode ? "TRUE" : "FALSE");
+  /* either inhibition already occurred, or we don't want to remove it yet */
+  if (power->priv->inhibited)
+    return;
 
+  xfce_screensaver_inhibit (power->priv->screensaver, presentation_mode);
   xfpm_update_blank_time (power);
+  xfpm_dpms_inhibit (power->priv->dpms, presentation_mode);
 }
 
 gboolean
@@ -1570,54 +1507,52 @@ xfpm_power_dbus_init (XfpmPower *power)
   GDBusConnection *bus = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, NULL);
   XfpmPowerManagement *power_dbus;
 
-  TRACE ("entering");
-
   power_dbus = xfpm_power_management_skeleton_new ();
   g_dbus_interface_skeleton_export (G_DBUS_INTERFACE_SKELETON (power_dbus),
                                     bus,
                                     "/org/freedesktop/PowerManagement",
                                     NULL);
 
-  g_signal_connect_swapped (power_dbus,
-                            "handle-shutdown",
-                            G_CALLBACK (xfpm_power_dbus_shutdown),
-                            power);
-  g_signal_connect_swapped (power_dbus,
-                            "handle-reboot",
-                            G_CALLBACK (xfpm_power_dbus_reboot),
-                            power);
-  g_signal_connect_swapped (power_dbus,
-                            "handle-hibernate",
-                            G_CALLBACK (xfpm_power_dbus_hibernate),
-                            power);
-  g_signal_connect_swapped (power_dbus,
-                            "handle-suspend",
-                            G_CALLBACK (xfpm_power_dbus_suspend),
-                            power);
-  g_signal_connect_swapped (power_dbus,
-                            "handle-can-reboot",
-                            G_CALLBACK (xfpm_power_dbus_can_reboot),
-                            power);
-  g_signal_connect_swapped (power_dbus,
-                            "handle-can-shutdown",
-                            G_CALLBACK (xfpm_power_dbus_can_shutdown),
-                            power);
-  g_signal_connect_swapped (power_dbus,
-                            "handle-can-hibernate",
-                            G_CALLBACK (xfpm_power_dbus_can_hibernate),
-                            power);
-  g_signal_connect_swapped (power_dbus,
-                            "handle-can-suspend",
-                            G_CALLBACK (xfpm_power_dbus_can_suspend),
-                            power);
-  g_signal_connect_swapped (power_dbus,
-                            "handle-get-on-battery",
-                            G_CALLBACK (xfpm_power_dbus_get_on_battery),
-                            power);
-  g_signal_connect_swapped (power_dbus,
-                            "handle-get-low-battery",
-                            G_CALLBACK (xfpm_power_dbus_get_low_battery),
-                            power);
+  g_signal_connect_object (power_dbus,
+                           "handle-shutdown",
+                           G_CALLBACK (xfpm_power_dbus_shutdown),
+                           power, G_CONNECT_SWAPPED);
+  g_signal_connect_object (power_dbus,
+                           "handle-reboot",
+                           G_CALLBACK (xfpm_power_dbus_reboot),
+                           power, G_CONNECT_SWAPPED);
+  g_signal_connect_object (power_dbus,
+                           "handle-hibernate",
+                           G_CALLBACK (xfpm_power_dbus_hibernate),
+                           power, G_CONNECT_SWAPPED);
+  g_signal_connect_object (power_dbus,
+                           "handle-suspend",
+                           G_CALLBACK (xfpm_power_dbus_suspend),
+                           power, G_CONNECT_SWAPPED);
+  g_signal_connect_object (power_dbus,
+                           "handle-can-reboot",
+                           G_CALLBACK (xfpm_power_dbus_can_reboot),
+                           power, G_CONNECT_SWAPPED);
+  g_signal_connect_object (power_dbus,
+                           "handle-can-shutdown",
+                           G_CALLBACK (xfpm_power_dbus_can_shutdown),
+                           power, G_CONNECT_SWAPPED);
+  g_signal_connect_object (power_dbus,
+                           "handle-can-hibernate",
+                           G_CALLBACK (xfpm_power_dbus_can_hibernate),
+                           power, G_CONNECT_SWAPPED);
+  g_signal_connect_object (power_dbus,
+                           "handle-can-suspend",
+                           G_CALLBACK (xfpm_power_dbus_can_suspend),
+                           power, G_CONNECT_SWAPPED);
+  g_signal_connect_object (power_dbus,
+                           "handle-get-on-battery",
+                           G_CALLBACK (xfpm_power_dbus_get_on_battery),
+                           power, G_CONNECT_SWAPPED);
+  g_signal_connect_object (power_dbus,
+                           "handle-get-low-battery",
+                           G_CALLBACK (xfpm_power_dbus_get_low_battery),
+                           power, G_CONNECT_SWAPPED);
 }
 
 static gboolean xfpm_power_dbus_shutdown (XfpmPower *power,
