@@ -92,6 +92,10 @@ enum
   PROP_LOGIND_HANDLE_HIBERNATE_KEY,
   PROP_LOGIND_HANDLE_LID_SWITCH,
   PROP_HEARTBEAT_COMMAND,
+
+  PROP_SHOW_PANEL_LABEL,
+  PROP_SHOW_PRESENTATION_INDICATOR,
+
   N_PROPERTIES
 };
 
@@ -616,6 +620,28 @@ xfpm_xfconf_class_init (XfpmXfconfClass *klass)
                                                          NULL, NULL,
                                                          NULL,
                                                          G_PARAM_READWRITE));
+
+  /**
+   * XfpmXfconf::show-panel-label
+   **/
+  g_object_class_install_property (object_class,
+                                   PROP_SHOW_PANEL_LABEL,
+                                   g_param_spec_int (SHOW_PANEL_LABEL,
+                                                     NULL, NULL,
+                                                     PANEL_LABEL_NONE,
+                                                     PANEL_LABEL_PERCENTAGE_AND_TIME,
+                                                     PANEL_LABEL_PERCENTAGE,
+                                                     G_PARAM_READWRITE));
+
+  /**
+   * XfpmXfconf::show-presentation-indicator
+   **/
+  g_object_class_install_property (object_class,
+                                   PROP_SHOW_PRESENTATION_INDICATOR,
+                                   g_param_spec_boolean (SHOW_PRESENTATION_INDICATOR,
+                                                         NULL, NULL,
+                                                         FALSE,
+                                                         G_PARAM_READWRITE));
 }
 
 static void
@@ -638,8 +664,8 @@ xfpm_xfconf_init (XfpmXfconf *conf)
   else
   {
     conf->priv->channel = xfconf_channel_get (XFPM_CHANNEL);
-    g_signal_connect (conf->priv->channel, "property-changed",
-                      G_CALLBACK (xfpm_xfconf_property_changed_cb), conf);
+    g_signal_connect_object (conf->priv->channel, "property-changed",
+                             G_CALLBACK (xfpm_xfconf_property_changed_cb), conf, 0);
   }
   xfpm_xfconf_load (conf);
 }
